"""Multi-Tooklit Error Dialog

Tries multiple python GUI libraries to display an error box
on the screen. No matter what is successful, the error will also
be sent to stdout. For GUI platforms, this can be a lot nicer than
opening a shell with errors dumped into it.

Call the "errorbox()" function with an error title and error message.

The different GUI libraries will leave the program in undefined
states, so this function will never return, but instead end the
program when the error has been dismissed. That makes this only
useful for errors, and not general message boxes.

Feel free to perhaps add some GUI handlers, as well as enhance
any that are here. They have all been tested on their appropriate
platforms.

There is even a decent pygame handler, if all else fails.

Use it to report things like missing modules (image, numeric, etc?).
Perhaps pygame raised an exception while initializing. This little
messagebox can sure be a lot nicer than a stack trace. ;]
"""
__author__ = "Peter Shinners"
__copyright__ = "Copyright (c) 2001 Peter Shinners"
__license__ = "Public Domain"
__version__ = "1.0"

import sys




def errorbox(title, message):
    "attempt to error with a gui"
    global handlers
    __stdout(title, message)
    for e in handlers:
        try:
            e(title, message)
            break
        except (ImportError, NameError): pass
    raise SystemExit



def __wxpython(title, message):
    from wxPython.wx import *
    class LameApp(wxApp):
        def OnInit(self): return 1
    app = LameApp()
    dlg = wxMessageDialog(None, message, title, wxOK|wxICON_EXCLAMATION)
    dlg.ShowModal()
    dlg.Destroy()


def __gtk(title, message):
    import GtkExtra
    GtkExtra.message_box(title, message, ['Ok'])


def __tkinter(title, message):
    import Tkinter, tkMessageBox
    Tkinter.Tk().wm_withdraw()
    tkMessageBox.showerror(title, message)


def __pygame(title, message):
    import os
    try:
        import pygame, pygame.font
        from pygame.locals import *
        pygame.quit() #clean out anything running
        pygame.display.init()
        pygame.font.init()
        screen = pygame.display.set_mode((460, 140))
        pygame.display.set_caption(title)
        font = pygame.font.Font(None, 18)
        foreg = 0, 0, 0
        backg = 200, 200, 200
        liteg = 255, 255, 255
        ok = font.render('Ok', 1, foreg)
        screen.fill(backg)
        okbox = ok.get_rect().inflate(20, 10)
        okbox.centerx = screen.get_rect().centerx
        okbox.bottom = screen.get_rect().bottom - 10
        screen.fill(liteg, okbox)
        screen.blit(ok, okbox.inflate(-20, -10))
        pos = [20, 20]
        for text in message.split('\n'):
            msg = font.render(text, 1, foreg)
            screen.blit(msg, pos)
            pos[1] += font.get_height()

        pygame.display.flip()
        while 1:
            e = pygame.event.wait()
            if e.type == QUIT or e.type == MOUSEBUTTONDOWN or \
                       (e.type == KEYDOWN and e.key in (K_ESCAPE, K_SPACE, K_RETURN)):
                break
        pygame.quit()
    except pygame.error:
        raise ImportError


def __stdout(title, message):
    text = 'ERROR: ' + title + '\n' + message
    print text

handlers = __gtk, __tkinter, __wxpython, __pygame


#test the error box
if __name__ == '__main__':
    errorbox("Testing", "This is only a test.\nHad this been " + \
          "a real emergency, you would be very afraid." )



"""
This is full-screen dialog box that allows the user to select an
item from a very large number of alternatives. Up to 165 items
are shown per page, and the user can advance forward or backward
through the list page by page. Selection is performed with the
left mouse button. The instructions, though printed to the
console instead of displayed on the screen, are simple and clear.

Takes a python list with a virtually infinite amount of items
(up to 16 chars long). If a list name is greater than 16
characters the rest of the chars will be truncated and
replaced with a tilde (~). and prints them in a Pygame
window 165 items at at time (in 640x480 resolution. Spacebar
to go to the next page, backspace to go to the previous page.
Left click an item to call a function on its name.
"""
__author__ = "Ryan Kulla"
__copyright__ = "Copyright (c) 2001 Ryan Kulla"
__license__ = "Public Domain"
__version__ = "1.0"


import pygame, pygame.font
from pygame.locals import *
from string import replace
import os.path

def command_file_master(screen, items):
    font = pygame.font.Font(None, 15)
    (screen_pause, place, marker) = (0, 0, 0)
    x = []
    if len(items) == 0:
            return (items, x, 1, place, marker)
    items.sort()
    for count in range(len(items)):
        #get rid of any newlines in the list names
        items[count] = replace(items[count], "\n", "")
    while 1:
            event = pygame.event.poll()
            if screen_pause == 1:
                    while 1:
                            event = pygame.event.wait()
                            cursor = pygame.mouse.get_pos()
                            if event.type == QUIT:
                                    raise SystemExit
                            if event.type == MOUSEBUTTONDOWN and\
                            pygame.mouse.get_pressed()[0]:
                                    for item in x:
                                            if item[0].collidepoint(cursor):
                                                    print 'you clicked ', item[1]
                            if event.type == KEYDOWN and event.key == K_SPACE:
                                    print 'space bar hit'
                                    if not place >= len(items):
                                            screen_pause = 0
                                            marker = 0
                                            x = []
                                            break
                            if event.type == KEYDOWN and event.key == K_BACKSPACE:
                                    print 'backspace hit'
                                    if ((place - marker) > 0):
                                            screen.fill(0)
                                            pygame.display.flip()
                                            screen_pause = 0
                                            place -= (165 + marker)
                                            marker = 0
                                            x = []
                                            break
            (items, x, screen_pause, place, marker) = file_master(screen,\
            font, items, place, marker, x)
            pygame.time.delay(5)


def file_master(screen, font, items, place, marker, x):
    screen.fill((0, 0, 0, 0))
    pygame.display.update()

    font_height = font.size(items[0])[1]
    screen_height = screen.get_height()
    max_file_width = 116
    name_max = 16 # how many maximum characters a list name can be
    line = 65 # leave room at top of screen for other stuff
    col = 30 # where to start the first column
    count = 0
    for name in items[place:]:
        count += 1
        place += 1
        marker += 1
        if count >= 165 or place >= len(items):
            ren_name = os.path.basename(name)
            if len(ren_name) > name_max:
                    ren_name = ren_name[:name_max] + '~'
            ren = font.render(ren_name, 1, (255, 255, 255), (0, 0, 0))
            ren_rect = ren.get_rect()
            ren_rect[0] = col
            ren_rect[1] = line
            x.append((ren_rect, name))
            screen.blit(ren, ren_rect)
            pygame.display.update(ren_rect)
            print 'space for next page, backspace for last page'
            return (items, x, 1, place, marker)
        ren_name = os.path.basename(name)
        if len(ren_name) > name_max:
            ren_name = ren_name[:name_max] + '~'
        ren = font.render(ren_name, 1, (255, 255, 255), (0, 0, 0))
        ren_rect = ren.get_rect()
        ren_rect[0] = col
        ren_rect[1] = line
        x.append((ren_rect, name))
        screen.blit(ren, ren_rect)
        line += 12
        if (line + font_height) >= (screen_height - 15):
            line = 65
            col += max_file_width
        pygame.display.update(ren_rect)
    return (items, x, 0, place, marker)


def main():
    pygame.init()
    screen = pygame.display.set_mode((640, 480))
    print 'hit i to start'
    while 1:
        event = pygame.event.wait()
        if event.type == QUIT:
            break
        if event.type == KEYDOWN and event.key == K_i:
            items = os.listdir('.')
            command_file_master(screen, items)
    pygame.quit()


if __name__=='__main__': main()



""" Text Input GUI
This module presents a simple text box and blocks until
a string is entered. The prompt ("Name" in the picture) is
supplied when the ask function is called, and the user is
able to use the backspace key to edit their input.

A program to get user input, allowing backspace etc shown
in a box in the middle of the screen. Called by:
  import inputbox
  answer = inputbox.ask(screen, "Your name")

Only near the center of the screen is blitted to
"""
__author__ = "Timothy Downs"
__copyright__ = "Copyright (c) 2002 Timothy Downs"
__license__ = "Public Domain"
__version__ = "1.0"


import pygame
from pygame.locals import *


def display_box(screen, message, size=20, fgcolor=(255,255,255), bgcolor=(0,0,0)):
    "Print a message in a box in the middle of the screen"
    fontobject = pygame.font.Font(None, size)
    area = Rect(0, 0, 200, size+6)
    area.center = screen.get_rect().center
    pygame.draw.rect(screen, bgcolor, area, 0)
    pygame.draw.rect(screen, fgcolor, area, 1)
    oldclip = screen.get_clip()
    screen.set_clip(area.inflate(-4, -4))
    if len(message):
        text = fontobject.render(message, 1, fgcolor, bgcolor)
        screen.blit(text, area.move(5, 4))
    screen.set_clip(oldclip)
    pygame.display.update(area)


def ask(screen, question):
    "ask(screen, question) -> answer"
    pygame.font.init()
    current_string = ''
    display_box(screen, question + ":")
    while 1:
        event = pygame.event.wait()
        if event.type == QUIT:
            break
        if event.type != KEYDOWN:
            continue
        if event.key == K_BACKSPACE:
            current_string = current_string[:-1]
        elif event.key == K_RETURN:
            break
        elif event.unicode:
            current_string += event.unicode

        display_box(screen, question + ": " + current_string)

    return current_string

if __name__ == '__main__':
    screen = pygame.display.set_mode((320,240))
    print "'%s' was entered" % ask(screen, "Name")



"""Code Profiling Class

profile.Profile sub-class using pygame.time.get_ticks.

The standard profile module uses the python timer; this code
replaces it with the pygame.time timer. Note that, as with the
standard profiler, machine-specific calibration may result in
more accurate measurements - check the profile.py module
documentation for more information.
"""
__author__ = "Kevin Turner"
__copyright__ = "Copyright (c) 2001 Kevin Turner"
__license__ = "Public Domain"
__version__ = "1.0"



import profile, pstats
import pygame

def f8_i(x):
    """using an integer clock, so lose the floating-point formatting."""

    return string.rjust(`x`,8)

pstats.f8 = f8_i

class PygameProfile(profile.Profile):
    """Profile sub-class using pygame.time.get_ticks.

    Seems to be pretty fast and precise.
    """

    def __init__(self):
        profile.Profile.__init__(self, pygame.time.get_ticks)

    def trace_dispatch_i(self, frame, event, arg):
        t = self.timer() - self.t # - 0 # Calibration constant
        # (but this needs no calibration constant on my machine)

        if self.dispatch[event](frame,t):
            self.t = self.timer()
        else:
            self.t = self.timer() - t # put back unrecorded delta
        return


def example():
    pr=PygameProfile()
    pr.run('function_to_profile(with, "args", as_needed)')
    pr.dump_stats('my_function.prof')

    # To display:

    # Also could be pstats.Stats(pr), if you still have the
    # Profile object around at this point.
    s = pstats.Stats('my_function.prof')

    s.strip_dirs().sort_stats('time')
    s.print_stats()
